module www
   use iso_fortran_env
   implicit none
   private
   character(len=1024) :: buffer

   public :: response_header, request, write_index
contains

   subroutine response_header(length)
      integer, intent(in) :: length

      write (output_unit, '(A)') "HTTP/1.1 200 OK"
      write (output_unit, '(A)') "Server: fortran-micro-httpd"
      write (output_unit, '(A)') "Content-Type: text/html; charset=UTF-8"
      write (output_unit, '(A)', advance='no') "Content-Length: "
      write (output_unit, *) length
      write (output_unit, *)
   end subroutine response_header

   subroutine request(post_content, post_content_length)
      character, dimension(:), allocatable, intent(out) :: post_content
      integer, intent(out) :: post_content_length
      character(len=:), allocatable :: client_request
      integer :: content_length, err, isize, cr_index, i

      client_request = ''
      post_content_length = 0
      read_client_request: do
         read (input_unit, iostat=err, fmt='(a)', advance='no', size=isize) buffer
         if (isize .eq. 0) exit read_client_request

         cr_index = index(buffer(:isize), "Content-Length: ", back=.true.)
         if (cr_index .ne. 0) then
            cr_index = index(buffer(:isize), ": ")
            read (buffer(cr_index + 2:isize), *) content_length
         end if

         client_request = client_request//buffer(:isize)//new_line('(A)')
      end do read_client_request

      if (content_length .gt. 0) then
         post_content_length = content_length
         allocate (post_content(content_length))
         do i = 1, content_length
            read (input_unit, iostat=err, fmt='(a)', advance='no', size=isize) post_content(i)
         end do
      end if
   end subroutine request

   subroutine write_index(filename)
      character(len=32), intent(in) :: filename
      character, dimension(:), allocatable  :: index_html
      integer :: filesize, err, i

      open (unit=11, file=filename, status='old', access='stream')
      inquire (unit=11, size=filesize)

      allocate (index_html(filesize))

      do i = 1, filesize
         read (11, pos=i) index_html(i)
      end do
      close (unit=11)

      call response_header(filesize)
      write (output_unit, *) index_html

      if (allocated(index_html)) deallocate (index_html, stat=err)
      if (err /= 0) print *, "index_html: Deallocation request denied"
   end subroutine write_index

end module www
